"use strict";
var field_model_1 = require("./field.model");
var enums_model_1 = require("./enums.model");
var Entity = (function () {
    function Entity() {
        this.properties = {};
    }
    Entity.prototype.setDefaults = function () {
        var _this = this;
        if (!this.properties)
            return;
        Object.keys(this.properties).forEach(function (property) {
            var description = _this.getPropertyDescription(property);
            if (description.initialValue)
                _this.setPropertyValue(property, description.initialValue);
            else
                _this.setPropertyValue(property, null);
        });
    };
    Entity.prototype.propertyExists = function (property) {
        return this.hasOwnProperty(property) || (property in this);
    };
    Entity.prototype.propertyDescribed = function (property) {
        return (property in this.properties);
    };
    Entity.prototype.getPropertyValue = function (property) {
        return this[property];
    };
    Entity.prototype.setPropertyValue = function (property, value) {
        this[property] = value;
    };
    Entity.prototype.getPropertyDescription = function (property) {
        return this.properties[property];
    };
    Entity.fromMap = function (map) {
        var name = this.toString();
        var classname = name.substring(name.indexOf(' ') + 1, name.indexOf('()'));
        var obj = window[classname];
        if (map.id)
            obj.id = map.id;
        Object.keys(map).forEach(function (property) {
            obj[property] = map[property];
        });
        return obj;
    };
    Entity.prototype.isNew = function () { return this.id && this.id != null; };
    Entity.prototype.isPropertyValid = function (property) {
        if (this.propertyExists(property) && this.propertyDescribed(property))
            try {
                if (field_model_1.FieldDefinition.isValid(this.getPropertyValue(property), this.getPropertyDescription(property), true)) {
                    this.getPropertyDescription(property).error.removeError();
                    return true;
                }
            }
            catch (e) {
                this.getPropertyDescription(property).error.setMessage(e.message);
                return false;
            }
        return false;
    };
    Entity.prototype.isValid = function () {
        var _this = this;
        var valid = false;
        Object.keys(this.properties).forEach(function (property) {
            valid = valid && _this.isPropertyValid(property);
        });
        return valid;
    };
    Entity.prototype.toJSON = function () {
        var _this = this;
        var json = {};
        Object.keys(this.properties).forEach(function (property) {
            if (_this.propertyExists(property)) {
                var description = _this.getPropertyDescription(property);
                var key = description.jsonKey ? description.jsonKey : property;
                var value = _this.getPropertyValue(property);
                // If string and has empty value, threath like a null
                if (description.type == enums_model_1.FieldType.String) {
                    if (value && value == '')
                        value = null;
                }
                if (value)
                    json[key] = value;
                else {
                    if (description.required)
                        console.log('Field ' + property + ' is null but is required. Something went wrong in the validation or in the configuration?');
                }
            }
        });
        return JSON.stringify(json);
    };
    Entity.prototype.propertyList = function () {
        return Object.keys(this.properties);
    };
    return Entity;
}());
exports.Entity = Entity;
//# sourceMappingURL=entity.model.js.map