"use strict";
var enums_model_1 = require("./enums.model");
var input_model_1 = require("./input.model");
var FieldError = (function () {
    function FieldError() {
    }
    FieldError.prototype.hasError = function () { return (this.message != null) && this.message != ''; };
    FieldError.prototype.removeError = function () { this.message = null; };
    FieldError.prototype.setMessage = function (message) { this.message = message; };
    FieldError.prototype.getMessage = function () { return this.message; };
    return FieldError;
}());
exports.FieldError = FieldError;
var FieldDefinition = (function () {
    function FieldDefinition() {
        this.required = false;
        this.error = new FieldError();
        this.showInList = true;
        this.input = new input_model_1.InputDefinition();
    }
    FieldDefinition.prototype.hasError = function () { return this.error.hasError(); };
    FieldDefinition.prototype.getError = function () { return this.error.getMessage(); };
    FieldDefinition.fromMap = function (map) {
        var definition = new FieldDefinition();
        Object.keys(map).forEach(function (property) {
            definition[property] = map[property];
        });
        return definition;
    };
    FieldDefinition.isValid = function (value, definition, throwException) {
        if (throwException === void 0) { throwException = false; }
        if (definition.type == enums_model_1.FieldType.String)
            return this.isStringValid(value, definition, throwException);
        if (definition.type == enums_model_1.FieldType.Number)
            return this.isNumberValid(value, definition, throwException);
        if (definition.type == enums_model_1.FieldType.Boolean)
            return true;
        // TODO: Add FieldType.Coordinates check
        if (throwException)
            throw new TypeError('Field type not recognized: ' + definition.type);
        else
            return false;
    };
    FieldDefinition.isStringValid = function (value, definition, throwException) {
        if (throwException === void 0) { throwException = false; }
        if (!value) {
            if (throwException && definition.required)
                throw new Error('Required');
            else
                return !definition.required;
        }
        if (definition.minLenght && definition.maxLength) {
            var valid = (value.length >= definition.minLenght && value.length <= definition.maxLength);
            if (!valid && throwException)
                throw new RangeError('Length between ' + definition.minLenght + ' and ' + definition.maxLength);
            return valid;
        }
        if (definition.minLenght) {
            var valid = (value.length >= definition.minLenght);
            if (!valid && throwException)
                throw new RangeError('Min length ' + definition.minLenght);
            return valid;
        }
        if (definition.maxLength) {
            var valid = (value.length <= definition.maxLength);
            if (!valid && throwException)
                throw new RangeError('Max length ' + definition.maxLength);
            return valid;
        }
        return true;
    };
    FieldDefinition.isNumberValid = function (value, definition, throwException) {
        if (throwException === void 0) { throwException = false; }
        if (!value) {
            if (throwException && definition.required)
                throw new Error('Required');
            else
                return !definition.required;
        }
        if (definition.minValue && definition.maxValue) {
            var valid = (value >= definition.minValue && value <= definition.maxValue);
            if (!valid && throwException)
                throw new RangeError('Use number between ' + definition.minValue + ' and ' + definition.maxValue);
            return valid;
        }
        if (definition.minValue) {
            var valid = (value >= definition.minValue);
            if (!valid && throwException)
                throw new RangeError('Use number > ' + definition.minValue);
            return valid;
        }
        if (definition.maxValue) {
            var valid = (value <= definition.maxValue);
            if (!valid && throwException)
                throw new RangeError('Use number < ' + definition.maxValue);
            return valid;
        }
        return true;
    };
    return FieldDefinition;
}());
exports.FieldDefinition = FieldDefinition;
//# sourceMappingURL=field.model.js.map